#include "analysis.hpp"

//#define TINYEXR_IMPLEMENTATION
//#define TINYEXR_USE_MINIZ 0
//#include "tinyexr.h"

RenderAnalysis::RenderAnalysis(QDir sequence) {
	this->sequence = sequence;
	frameTimes = new QLineSeries();
	frameTimes->setName(getSequenceName());
	totalRenderTime = 0;
}

RenderAnalysis::~RenderAnalysis() {
	delete frameTimes;
}

bool RenderAnalysis::analyze() {
	QStringList imageFiles = sequence.entryList(sequence.nameFilters(), QDir::Files, QDir::Name);
	QStringListIterator iterator(imageFiles);
	bool notAllFailed = false;
	while (iterator.hasNext()) {
		notAllFailed = notAllFailed | analyzeFrame(sequence.absoluteFilePath(iterator.next()));
	}
	printf("Total render time for series: %s or %f\n", secondsToTime(totalRenderTime).toStdString().c_str(), totalRenderTime);
	return notAllFailed;
}

float RenderAnalysis::timeToSeconds(QString time) {
	float seconds = 0;
	QStringList sections = time.split(':');

	seconds += sections.last().toFloat(); //seconds
	sections.removeLast();
	seconds += sections.last().toInt() * 60; //minutes
	sections.removeLast();
	if (sections.isEmpty()) return seconds;
	seconds += sections.last().toInt() * 60 * 60; //hours
	return seconds;
}

QString RenderAnalysis::secondsToTime(float seconds) {
	int remaining = seconds;
	int secs = remaining % 60;
	remaining -= secs;
	int mins = (remaining % 3600) / 60;
	remaining -= mins * 60;
	int hours = (remaining % 86400) / 3600;
	remaining -= hours * 3600;
	int days = remaining / 60 / 60 / 24;

	if (days > 0)
		return QString::asprintf("%dd %dh %dm %ds", days, hours, mins, secs);
	else if (hours > 0)
		return QString::asprintf("%dh %dm %ds", hours, mins, secs);
	else
		return QString::asprintf("%dm %ds", mins, secs);
}

bool RenderAnalysis::analyzeFrame(QString filename) {
	QFile file(filename);
	if (!file.open(QIODevice::ReadOnly)) {
		printf("%s: could not open file!!!!\n", filename.toStdString().c_str());
		return false;
	}
	QDataStream fileStream(&file);
	fileStream.setByteOrder(QDataStream::LittleEndian);

	int magic = 0;
	fileStream >> magic;
	if (magic != 20000630) {
		printf("%s: Magic %d does not match!\n", filename.toStdString().c_str(), magic);
		return false;
	}
	
	//TODO: check the exr version
	QByteArray versionField = file.read(4);

	// things we're going to extract from the attributes
	float renderTime = 0.0f;
	int frameNum = 0;

	bool endOfAttributes = false;
	while (!endOfAttributes) {
		QByteArray attributeName;
		int attributeSize = 0;
		QByteArray attributeType;
		QByteArray attributeValue;

		bool endOfString = false;
		while(!endOfString) {
			char newByte;
			fileStream.readRawData(&newByte, 1);
			if (newByte == '\0') {
				endOfString = true;
			} else {
				attributeName.append(newByte);
			}
		}

		// empty attribute name means no more attributes, end of header section
		if (attributeName.size() == 0) {
			endOfAttributes = true;
			break;
		}

		endOfString = false;
		while(!endOfString) {
			char newByte;
			fileStream.readRawData(&newByte, 1);
			if (newByte == '\0') {
				endOfString = true;
			} else {
				attributeType.append(newByte);
			}
		}

		fileStream >> attributeSize;

		//read the attribute value
		attributeValue = file.read(attributeSize);

#ifdef _DEBUG
		printf("%s (%s): %s\n", attributeName.data(), attributeType.data(), attributeValue.data());
#endif

		if (QString(attributeName) == "RenderTime") {
			renderTime = timeToSeconds(QString(attributeValue));
#ifdef _DEBUG
			printf("FOUND ATTRIBUTE: render time is %f seconds\n", renderTime);
#endif
		}
		if (QString(attributeName) == "Frame") {
			frameNum = attributeValue.toInt();
#ifdef _DEBUG
			printf("FOUND ATTRIBUTE: frame number is %d\n", frameNum);
#endif
		}
	}
	printf("%s: Frame %d rendered in %.2f seconds\n", filename.toStdString().c_str(), frameNum, renderTime);

	// append this to the line chart data
	frameTimes->append(frameNum, renderTime);
	
	totalRenderTime += renderTime;

	fileStream.setDevice(nullptr);
	file.close();

	return true;
}

QLineSeries* RenderAnalysis::getFrameTimeSeries() {
	return frameTimes;
}

QPieSlice RenderAnalysis::getRenderTimePieSlice() {
	return QPieSlice(getSequenceName(), totalRenderTime);
}

QString RenderAnalysis::getSequenceName() {
	return sequence.nameFilters().at(0);
}

QString RenderAnalysis::getSequencePath() {
	return sequence.absoluteFilePath(getSequenceName());
}

float RenderAnalysis::getTotalRenderTime() {
	return totalRenderTime;
}

QString RenderAnalysis::getTotalRenderTimeText() {
	return secondsToTime(totalRenderTime);
}
