#include "analyzer.hpp"

RenderAnalyzer::RenderAnalyzer(QWidget* parent, QListView* listView) {
	this->parent = parent;
	this->listView = listView;

	analysisList = new QList<RenderAnalysis*>();

	frameTimeChart = new QChart();
	frameTimeChart->setTitle(tr("Render time per frame"));
	frameTimeChart->setTheme(QChart::ChartThemeDark);
	frameTimeChart->setAnimationOptions(QChart::SeriesAnimations);

	renderTimePieSeries = new QPieSeries();
	renderTimePieSeries->setPieSize(0.5);
	renderTimeChart = new QChart();
	renderTimeChart->setTitle(tr("Render time per sequence"));
	renderTimeChart->setTheme(QChart::ChartThemeDark);
	renderTimeChart->setAnimationOptions(QChart::SeriesAnimations);
	renderTimeChart->addSeries(renderTimePieSeries);
	renderTimeChart->legend()->setVisible(false);
}

RenderAnalyzer::~RenderAnalyzer() {
	QListIterator<RenderAnalysis*> analysisListIterator(*analysisList);
	while (analysisListIterator.hasNext()) {
		delete analysisListIterator.next();
	}
	delete analysisList;
	delete frameTimeChart;
	delete renderTimePieSeries;
	delete renderTimeChart;
}

void RenderAnalyzer::addRender() {
	QFileInfo selectedImage(QFileDialog::getOpenFileName(parent, tr("Open Sequence"), "", tr("EXR Images (*.exr)")));

	// extract filename pattern from selected image
	QString filterPattern = selectedImage.fileName();
	filterPattern.replace(QRegularExpression("[0-9]+.exr"), "*.exr");

	QString directory = selectedImage.canonicalPath();

	printf("Analyzing %s in directory %s...\n", filterPattern.toStdString().c_str(), directory.toStdString().c_str());

	QDir sequence(directory, filterPattern);
	RenderAnalysis* analysis = new RenderAnalysis(sequence);

	if (analysis->analyze()) {
		beginInsertRows(QModelIndex(), rowCount(), rowCount());

		analysisList->append(analysis);
		frameTimeChart->addSeries(analysis->getFrameTimeSeries());
		frameTimeChart->createDefaultAxes();
		renderTimePieSeries->append(
				"<b>(" + analysis->getTotalRenderTimeText() + ")</b><br/>" + analysis->getSequenceName(),
				analysis->getTotalRenderTime());

		QPieSlice *newSlice = renderTimePieSeries->slices().at(renderTimePieSeries->slices().size() - 1);
		newSlice->setLabelVisible(true);

		QValueAxis* hAxis = (QValueAxis*)frameTimeChart->axes(Qt::Horizontal).at(0);
		QValueAxis* vAxis = (QValueAxis*)frameTimeChart->axes(Qt::Vertical).at(0);

		hAxis->setTitleText("Frame number");
		hAxis->setLabelFormat("%04dF");
		vAxis->setTitleText("Render Time (seconds)");
		vAxis->setLabelFormat("%.2fs");

		endInsertRows();
		emit renderAdded();
	} else {
		delete analysis;
	}
}

void RenderAnalyzer::removeSelectedRender() {
	QModelIndexList selectedIndexes = listView->selectionModel()->selectedIndexes();
	if (selectedIndexes.size() == 0) {
		return;
	}

	QModelIndex selected = selectedIndexes.at(0);
	RenderAnalysis* analysis = analysisList->at(selected.row());
	QLineSeries* frameTimeSeries = analysis->getFrameTimeSeries();
	QPieSlice* renderTimeSlice = renderTimePieSeries->slices().at(selected.row());

	beginRemoveRows(QModelIndex(), selected.row(), selected.row());

	renderTimePieSeries->remove(renderTimeSlice);

	frameTimeChart->removeSeries(frameTimeSeries);
	frameTimeChart->createDefaultAxes(); // recalculate axes

	analysisList->removeAt(selected.row());
	delete analysis;

	endRemoveRows();
}

void RenderAnalyzer::prefixChanged(QString prefix) {
	nextPrefix = prefix;
}

QChart* RenderAnalyzer::getFrameTimeChart() {
	return frameTimeChart;
}

QChart* RenderAnalyzer::getRenderTimeChart() {
	return renderTimeChart;
}

int RenderAnalyzer::rowCount(const QModelIndex &parent) const {
	if (parent.column() < 1) {
		return analysisList->size();
	} else {
		return 0;
	}
}

QVariant RenderAnalyzer::data(const QModelIndex &index, int role) const {
	if (index.column() != 0) {
		return QVariant();
	}

	if (role == Qt::DisplayRole) {
		return analysisList->at(index.row())->getSequencePath();
	} else {
		return QVariant();
	}
}
